/************************************************************************/
/*                                                                      */
/*   Program Name :  m5tok5                                             */
/*                                                                      */
/*     M5tok5 is a general pupose Mark-5 to K5 file format converter    */
/*       both parity and non-parity modes are supported                 */
/*       This is faster than m5tok5R by about two times.                */
/*                                                                      */
/*     Enviroment variables                                             */
/*           M5DIR      ---   default directory for Mark5 data          */
/*           M5VEX      ---   default directory for Mark5 vex file      */
/*                                                                      */
/*            Ver 1.00  2002-10-20                                      */
/*            Ver 2.03  2005-01-13                                      */
/*            Ver 2.10  2005-06-17  restructuring                       */
/*            Ver 2.11  2006-09-27                                      */
/*            Ver 2.20  2007-03-30  more robust against malfunctioned   */
/*                                  mark5 data file                     */
/*            Ver 2.21  2007-07-18  file name size increased (80->256)  */
/*            Ver 2.30  2008-01-08  debug for 2008 problem which is     */
/*                                  a problem unable to convert Mark5   */
/*                                  data time-labeled 2008              */
/*            Ver 2.31  2008-02-19  options '--help' and '--version'    */
/*                                  were added                          */
/*            Ver 2.32  2011-02-09                                      */
/*                                                                      */
/*      Copyright (c) 2002-2011 T.Kondo/NICT All Right Reserved         */
/*                                                                      */
/************************************************************************/
/*

G:\jive\f03x1_5.mc -g 1 -i G:\jive\f03x1_info.txt
     2002-10-17  -- development started
     2004-09-23  -- Version up to 2.00  
     2004-09-24  -- debug in VLBA mode  
     2004-11-14  -- debug in bit order in case of 2bit sampling  
     2005-01-13  -- minor change in station selection method  
     2005-06-17  -- update due to library restructuring  
     2006-09-27  -- update due to library restructuring  
     2007-03-30  -- more robust against malfunctioned mark5 data file
     2007-07-18  -- file name size was increased from 80 chars to 256 chars
     2008-01-08  -- debug for 2008 problem which is a problem unable to convert Mark5
                    data time-labeled 2008
     2008-02-19  -- options '--help' and '--version'  were added 
     2011-02-09  -- scanf("%s",&XX) was changed to scanf("%s",XX) when XX is 
                    declared as like "char XX[20]" to avoid warning at Ubuntu 10.10
                    scanf(...) was changed to like i=scanf(...) to avoid warning
                    at Ubuntu 10.10

 Mode 1
   how to run
          m5tok5 mk5_filename [options]
    
     where   mk5filename -- Mark 5 data file name
              options (any order)
               -c channel    -- pickup channel (1-16) for 1ch conversion mode
                                 (default all group and 4ch mode)
               -g group      -- pickup group (1-4) for 4 ch conversion mode
                                 (omitted when -c option is set)
                                 (default all group)
               -i info_file  -- infomation file name that contains
                                  track vs channel table
                                  bit_psition vs channel table
                                  group# vs channel table
                                  (default "m5tok5info.txt")
                                  "-i -info_file"  means info_file named info_file creation
                                  "-i make" means default info_file creation
               -o k5name     -- K5 file name to be created
                                 (default : see below)
               -d k5dir      -- directory for k5 file out
                                 (default  : mark5 directory)
               -s strat_sec  -- offset time from data head (sec)
                                 (default :  0 )
               -p period     -- data period to convert (sec)
                                 (default : all)
               -r            -- reverse track order when fanout =2 or more
               -v vex_file   -- VEX file name to be read when "-i make" is set
               -sid stat_id  -- station ID when "-i make" is set
               -scan scan#   -- scan# for MODE get when "-i make" is set
                                 (default : 1)\n");
               -subp subpass -- set subpass at information file 
                                 creation mode (default is "A") 
               -odd          -- 32bit (4byte) shift in reading Mark5 data
                                 (this works only for 64 track mode)
               -monit        -- info_file monitor ON 

  or
  Mode 2
   how to run
       m5tok5 mk5_name k5_name bit1 [bit2 bit3 bit4 ... bitN] [options2]
    
      where  mk5_name -- Mark 5 data file name
             k5_name  -- K5 file name to be created
             bit1 ------ 1st pick up bit position (0- max track#)
             bit2 ------ 2nd pick up bit position (0- max track#)
               .
               .
             bitN ------ Nth pick up bit position (0- max track#)
                       note: N should be (K5 channels)*(AD bits)*(Mark5 Fanout)
                          example           4 ch         1 bit        2


             options2 (any order)
               -track ntrack  -- # of Mark5 track (8,16,32,64) (deflt=32)
               -fsampl fmhz   -- channel sampling frequency (MHz) (deflt=4)
               -adbit adbit   -- # of AD conversion bit (deflt=1)
               -nch numch     -- # of channels in K5 format (1 or 4) (deflt=4)
               -parity        -- with parity data 
               -noparity      -- non-parity data (deflt)
               -vlba          -- set VLBA mode (default mark IV mode)
               -s strat_sec   --  offset time from data head (sec)
                                  (default :  0 )
               -p period      --  data period to convert (sec)
                                  (default : all)
               -odd           -- 32bit (4byte) shift in reading Mark5 data
                                  (this works only for 64 track mode)

               --- hidden paramers for just in case ----
               -fanout fanout -- # of fanout (deflt=1)
                                 automatically set from # of pick-up bit position
               -nrzm          -- NRZM coding
               -nrzl          -- NRZL coding (deflt)
                                 if parity is set, NRZM is automatically set.
                                 if NRZM is set, parity is automatically set.                           

   
  Naming rule for K5 file created (MK5 : original mark5 name)
         4ch mode
            MK5.k5a ---- for group#1 in info_file (usually CH 01-04)
            MK5.k5b ---- for group#2 in info_file (usually CH 05-08)
            MK5.k5c ---- for group#3 in info_file (usually CH 09-12)
            MK5.k5d ---- for group#4 in info_file (usually CH 13-16)
         1ch mode
            MK5.k5-NN ---- where NN is channel number 01-16

  Enviroment variables
          M5DIR    ---   default path to Mark5 data
          M5VEX    ---   default path to Mark5 vex file


  Note:  When compiling on VC++, set stack size at least 4000000(4MB).
         otherwise results in runtime error.


*** sample run parameters ****

MODE 1

1. make a information file 
 case 1:  make a default information file (m5tok5info.txt)
  G:\IPVLBI\data\jiveexp3\d03l1_2.on -i make -v G:\IPVLBI\data\jiveexp3\d03l1.vex -sid On

 case 2:  make an information file named d03l1_on_info.txt
  G:\IPVLBI\data\jiveexp3\d03l1_2.on -i -G:\IPVLBI\data\jiveexp3\d03l1_on_info.txt -v G:\IPVLBI\data\jiveexp3\d03l1.vex -sid On

2. convert Mark5 file to K5 file
 case 1: using a default information file
  G:\IPVLBI\data\jiveexp3\d03l1_2.on -g 1

 case 2: using an information file named d03l1_info.txt
  I:\data\jiveexp3\d03l1_2.on -g 1 -i I:\data\jiveexp3\d03l1_on_info.txt

3. other examples for debugging purpose
  G:\rd0401\E085-1224 -i G:\rd0401\rd0401.Einfo.txt -g 1
  G:\jive\f03x1_5.ef -g 1 -i G:\jive\f03x1_info.txt
  G:\IPVLBI\data\evlbi3\288-1730 -i make -v G:\IPVLBI\data\evlbi3\2994.ovex
  G:\IPVLBI\data\evlbi3\288-1730 -g 1 -i G:\IPVLBI\data\evlbi3\evlbi3_info.txt
  G:\GSICHECK\K04242\242-0734 -g 1 -s 3 -i G:\GSICHECK\K04242\k04242info.txt
  G:\IPVLBI\data\evlbi3\O2880001.m5 -g 1 -i G:\IPVLBI\data\evlbi3\vlbainfo.txt
  G:\mark5\m5test.dat -g 1 -i G:\mark5\m5test_info.txt
  I:\data\GSIcheck\rdv71\test.m5 -i I:\data\GSIcheck\rdv71\m5tok5infoR71.txt -g 1

MODE 2
  G:/IPVLBI/data/evlbi3/288-1730 G:/IPVLBI/data/evlbi3/E2880001a.k5dat 16 2 18 4 -pa
  G:\IPVLBI\data\tsev6\2-178-1304.dat G:\IPVLBI\data\tsev6\2-178-1304.dat.k5at 2 3 4 5  -nopa -tr 16 -nch 4
  G:\IPVLBI\data\jiveexp3\d03l1_2.on G:\IPVLBI\data\jiveexp3\d03l1_2.on.k5at 
      32 40 48 56 33 41 49 57 34 42 50 58 35 43 51 59 36 44 52 60 37 45 53 
      61 38 46 54 62 39 47 55 63 -adbit 2 -nonparity -track 64 -fsampl 16
  G:\IPVLBI\data\evlbi3\O2880001.m5 G:\IPVLBI\data\evlbi3\k01.dat 0 1 2 3 -nch 4 -vlba -parity
*/

#define PROGRAM_VERSION "m5tok5 (Ver 2.32  2011-02-09)"

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libmk5tok5.h"  // library for Mark5 to K5 conversion

/* default conversion information file name set */
#define INFONAME_DFLT "m5tok5info.txt"   // information file default name
/* environment variable for Mark5 data default directory set */
#define M5DIR "M5DIR" // the environment variable for Mark5 data default directory
/* environment variable for Mark5 vex file default directory set */
#define M5VEX "M5VEX" // the environment variable for Mark5 vex file

/* default pass set */
#define DEFLT_PASS "A"   /* default pass in $TRACKS in a VEX file (added on 2004-06-29) */

/* default parameters for run mode 2 */
#define K5NUMCH  4
#define M5PARITY 0
#define M5VLBA   0
#define M5FANOUT 1
#define M5ADBIT  1
#define M5SAMPLE 4   // default sampling frequency in MHz
#define M5NTRACK 32

void param_get_m5tok5(int argc, char *argv[],m5k5runp_t *runp)                       
{
    int n,i,len;
    char cbuf[256];
    //1st param should be mark5 name
    if (argc>=2){
       strcpy(runp->mk5name,*(argv+1));
    } else {
       runp->mk5name[0]='\0';
    }
     /* check Mark5 file name is acompanied by directory or not */
    get_true_path(runp->mk5name, M5DIR, "./",cbuf);
    strcpy(runp->mk5name,cbuf);

    // initialize parameters

    /* common parameters */
    runp->ptype=1;
    runp->k5name[0]='\0';
    runp->k5name_given[0]='\0';
    runp->soffset=0;
    runp->period=0;
    runp->odd=0;

    /* mode 1 parameters */
    runp->channel=0;
    runp->group=0;
    runp->reverse=0;
    runp->makemode=0;
    runp->info_file[0]='\0';
    runp->outdir[0]='\0';
    runp->vex_file[0]='\0';
    runp->sid[0]='\0';
    runp->scan=1;
    runp->monit=0;
    strcpy(runp->dflt_pass, DEFLT_PASS);

    /* mode 2 parameters */
    runp->numch=K5NUMCH;
    runp->ntrack=M5NTRACK;
    runp->adbit =M5ADBIT;
    runp->parity=M5PARITY;
    runp->nrzm  =-1;
    runp->fanout=M5FANOUT;
    runp->vlba  =M5VLBA;
    runp->tot_track=0;
    runp->fmhz  =(double)M5SAMPLE;
    for(n=0; n<32; n++) runp->ibit[n]=0;


    /* check run parameter mode */
    if(argc>=3){
        strcpy(cbuf,*(argv+2) );
        if(cbuf[0] != '-') goto mode2;  // mode 2 type parameters
    }

    /*  mode 1 parameter analysis */
    n=3;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,"-i",2)==0){  // info file
            n++;
            if(n > argc) break;
            strcpy(runp->info_file,*(argv+n-1));
            if(runp->info_file[0]=='-'){
                runp->makemode=1;
                len=strlen(runp->info_file);
                for(i=0;i<len-1;i++) runp->info_file[i]=runp->info_file[i+1];
                runp->info_file[len-1]='\0';
            } else if(strncmp(runp->info_file,"make",4)==0){
                runp->makemode=1;
                runp->info_file[0]='\0';
            }
        }
        if(strlen(cbuf)==2 && strncmp(cbuf,"-o",2)==0){  // k5 name
            n++;
            if(n > argc) break;
            strcpy(runp->k5name_given,*(argv+n-1));
        }
        if(strncmp(cbuf,"-d",2)==0){  // out directory
            n++;
            if(n > argc) break;
            strcpy(runp->outdir,*(argv+n-1));
            /* env variable is substituted by real one */

            if((i=substitute_env_path_name(runp->outdir, cbuf))>0){
                strcpy(runp->outdir,cbuf);
            }

        }
        if(strncmp(cbuf,"-c",2)==0){  // channel define
            n++;
            if(n > argc) break;
            runp->channel = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-g",2)==0){  // group define
            n++;
            if(n > argc) break;
            runp->group = atoi(*(argv + n-1 ));
        }
        if(strlen(cbuf)==2 && strncmp(cbuf,"-s",2)==0){  // start offset
            n++;
            if(n > argc) break;
            runp->soffset = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-p",2)==0){  // period
            n++;
            if(n > argc) break;
            runp->period = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-r",2)==0){  // 
            runp->reverse=1;
        }
        if(strncmp(cbuf,"-v",2)==0){  // vex file
            n++;
            if(n > argc) break;
            strcpy(runp->vex_file,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-si",3)==0){  // station id
            n++;
            if(n > argc) break;
            strcpy(runp->sid,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-scan",5)==0){  // scan#
            n++;
            if(n > argc) break;
            runp->scan=atoi(*(argv + n-1 ));
            if(runp->scan < 1) runp->scan=1;
        }
        if(strncmp(cbuf,"-subp",5)==0){  // subpass
            n++;
            if(n > argc) break;
            strcpy(runp->dflt_pass,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-odd",4)==0){  // odd mode set
            runp->odd=1;
        }
        if(strncmp(cbuf,"-monit",2)==0){  // monitor mode on
            runp->monit=1;
        }
        n++;
    }
    goto exitend;

mode2:
    /* mode 2 parameter analysis */

    runp->ptype=2;
    if (argc>=3) strcpy(runp->k5name_given,*(argv+2));

    n=4;
    // bit position get
    while(1){
        if(argc >= n) {
           strcpy(cbuf,*(argv+ n-1) );
           //printf("%s\n",cbuf);
           if(cbuf[0] == '-') break;  // to option 2 parameters
           runp->ibit[runp->tot_track] = atoi(*(argv + n-1));
           runp->tot_track++;
        } else {
            goto exitend;
        }
        n++;
    }

    // option 2 parameters get
    while(n <= argc){
       strcpy(cbuf,*(argv+n-1));
       if(strncmp(cbuf,"-track",3)==0){  // track#
            n++;
            if(n > argc) break;
            runp->ntrack=atoi(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-fsampl",3)==0){  // fsampl
            n++;
            if(n > argc) break;
            runp->fmhz=atof(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-adbit",3)==0){  // adbit
            n++;
            if(n > argc) break;
            runp->adbit=atoi(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-fanout",3)==0){  // fanout
            n++;
            if(n > argc) break;
            runp->fanout=atoi(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-nch",3)==0){  // # of channels
            n++;
            if(n > argc) break;
            runp->numch=atoi(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-parity",3)==0){  // parity
           runp->parity=1;
       }
       if(strncmp(cbuf,"-noparity",3)==0){  // non-parity
           runp->parity=0;
       }
       if(strncmp(cbuf,"-nrzm",5)==0){  // NRZM
           runp->nrzm=1;
       }
       if(strncmp(cbuf,"-nrzl",5)==0){  // NRZL
           runp->nrzm=0;
       }
       if(strncmp(cbuf,"-vlba",3)==0){  // VLBA mode
           runp->vlba=1;
       }
       if(strlen(cbuf)==2 && strncmp(cbuf,"-s",2)==0){  // start offset
           n++;
           if(n > argc) break;
           runp->soffset = atoi(*(argv + n-1 ));
       }
       if(strlen(cbuf)==2 && strncmp(cbuf,"-p",2)==0){  // period
           n++;
           if(n > argc) break;
           runp->period = atoi(*(argv + n-1 ));
       }
       if(strncmp(cbuf,"-odd",4)==0){  // odd mode set
            runp->odd=1;
       }
       n++;
    }
exitend:;
}

void print_usage(void)
{
    char cw;
    printf("%s\n",PROGRAM_VERSION);
    printf("\n");
    printf("How to use (MODE 1)\n");
    printf("\n");
    printf("    m5tok5 mk5_filename [options]\n");
    printf("\n");
    printf("where   mk5_filename -- Mark 5 data file name\n");
    printf("      options (any order)\n");
    printf("        -c channel    -- pickup channel (1-16) for 1ch conversion mode\n");
    printf("                          (default all group and 4ch mode)\n");
    printf("        -g group      -- pickup group (1-4) for 4 ch conversion mode\n");
    printf("                          (omitted when -c option is set)\n");
    printf("                          (default all group)\n");
    printf("        -i info_file  -- infomation file name that contains\n");
    printf("                           track vs channel table\n");
    printf("                           bit_psition vs channel table\n");
    printf("                           group# vs channel table\n");
    printf("                           (default \"%s\")\n",INFONAME_DFLT);
    printf("                           \"-i -info_file\" means\n");
    printf("                             create infomation file named info_file\n");
    printf("                           \"-i make\" means create default info file\n");
    printf("        -o k5name     -- K5 file name to be created\n");
    printf("                          (default : see below)\n");
    printf("        -d k5dir      -- directory for k5 file out\n");
    printf("                          (default  : mark5 directory)\n");
    printf("        -s strat_sec  -- offset time from data head (sec)\n");
    printf("                          (default :  0 )\n");
    printf("        -p period     -- data period to convert (sec)\n");
    printf("                          (default : all)\n");
    printf("        -r            -- reverse track order when fanout =2 or more\n");
    printf("        -v vex_file   -- VEX file name to be read when \"information\n");
    printf("                              file creation mode\" is set\n");
    printf("        -sid stat_id  -- station ID when \"information file creation\n");
    printf("                              mode\" is set\n");
    printf("        -scan scan#   -- scan# for MODE get when \"information file\n");
    printf("                              creation mode\" is set (default : 1)\n");
    printf("        -subp subpass -- set subpass at information file\n");
    printf("                              creation mode (default is \"A\")\n");
    printf("        -odd          -- 32bit (4byte) shift in reading Mark5 data\n");
    printf("                          (this works only for 64 track mode)\n");
    printf("        -monit        -- info_file monitor ON\n");
    printf("\n");
    printf("   Naming rule for K5 file created (MK5 : original mark5 name)\n");
    printf("      4ch mode\n");
    printf("        MK5.k5a ---- for group#1 in info_file (usually ch01-04)\n");
    printf("        MK5.k5b ---- for group#2 in info_file (usually ch05-08)\n");
    printf("        MK5.k5c ---- for group#3 in info_file (usually ch09-12)\n");
    printf("        MK5.k5d ---- for group#4 in info_file (usually ch13-16)\n");
    printf("      1ch mode\n");
    printf("        MK5.k5-NN ---- where NN is channel number 01-16\n");
    printf("   Enviroment variables\n");
    printf("        %s    ---   default path to Mark5 data ( %s )\n",
        M5DIR,getenv(M5DIR));
    printf("        %s    ---   default path to Mark5 vex file ( %s )\n",
        M5VEX,getenv(M5VEX));

    printf(" Hit return key for another how to run");
    //scanf("%s",&cw);  // dummy read
    cw=getchar();  // dummy read to accept just return
    printf("\n");
    printf("How to use (MODE 2)\n");
    printf("\n");
    printf("    m5tok5 m5file k5file bit1 [bit2 bit3 .. bitN] [options2]\n");
    printf("\n");
    //printf("   This is valid only for 4MHz 1bit-sampling 32-track data\n");
    //printf("        with NRZM coding with parity.\n");
    printf("where  m5file ---- Mark-5 data file name to read\n");
    printf("       k5file ---- K-5 file name to be created\n");
    printf("       bit1   ---- 1st pick up bit position (0- max track#)\n");
    printf("       bit2   ---- 2nd pick up bit position (0- max track#)\n");
    printf("         .          .\n");
    printf("         .          .\n");
    printf("       bitN   ---- Nth pick up bit position (0- max track#)\n");
    printf("               note: N should be (K5 channels)*(AD bits)*(Mark5 Fanout)\n");
    printf("                     example           4 ch         1 bit        2\n");
    printf("      options2 (any order)\n");
    printf("        -track ntrack  -- # of Mark5 track (8,16,32,64) (deflt=%d)\n",M5NTRACK);
    printf("        -fsampl fmhz   -- channel sampling frequency (MHz) (deflt=%d)\n",M5SAMPLE);
    printf("        -adbit adbit   -- # of AD conversion bit (deflt=%d)\n",M5ADBIT);
    //printf("        -fanout fanout -- # of fanout (deflt=%d)\n",M5FANOUT);
    //printf("                          automatically set from # of pick-up bit position\n");
    printf("        -nch numch     -- # of channels in K5 format (1 or 4) (deflt=%d)\n",K5NUMCH);
    if(M5PARITY==0){
       printf("        -parity        -- with parity data \n");
       printf("        -noparity      -- non-parity data (deflt)\n");
    } else {
       printf("        -parity        -- with parity data (deflt)\n");
       printf("        -noparity      -- non-parity data\n");
    }
    //printf("        -nrzm          -- NRZM coding\n");
    //printf("        -nrzl          -- NRZL coding (deflt)\n");
    //printf("                          if parity is set, NRZM is automatically set.\n");
    //printf("                          if NRZM is set, parity is automatically set.\n");                        
    printf("        -vlba          -- set VLBA mode (default mark IV mode)\n");
    printf("        -s strat_sec   -- offset time from data head (sec)\n");
    printf("                          (default :  0 )\n");
    printf("        -p period      -- data period to convert (sec)\n");
    printf("                          (default : all)\n");
    printf("        -odd           -- 32bit (4byte) shift in reading Mark5 data\n");
    printf("                          (this works only for 64 track mode)\n");
    printf("\n");
}

void monit_runp(m5k5runp_t *runp)
// run parameter monitor
{
    int n;

    switch(runp->ptype){
    case 1:
       printf(" ---------- RUN CONDITION ---- (Mode = %d )-------\n",runp->ptype);
       printf("   mk5name   : %s\n",runp->mk5name);
       printf("   k5name    : %s\n",runp->k5name_given);
       printf("   outdir    : %s\n",runp->outdir);
       printf("   infofile  : %s\n",runp->info_file);
       printf("   channel   : %d\n",runp->channel);
       printf("   group     : %d\n",runp->group);
       printf("   soffset   : %d\n",runp->soffset);
       printf("   period    : %d\n",runp->period);
       printf("   odd       : %d\n",runp->odd);
       if(runp->vex_file[0] != '\0')
          printf("   vex_file  : %s\n",runp->vex_file);
       if(runp->sid[0] != '\0' && runp->sid[0] !='-')
          printf("   station_id: %s\n",runp->sid);
       if(runp->makemode==1){
          printf("   scan# for MODE get : %d\n",runp->scan);
          printf("   subpass to select  : %s\n",runp->dflt_pass);
       }
       printf(" -------------------------------------\n");
       break;
    case 2:
       printf(" ---------- RUN CONDITION ---- (Mode = %d )-------\n",runp->ptype);
       printf("   mk5name   : %s\n",runp->mk5name);
       printf("   k5name    : %s\n",runp->k5name_given);
       printf("   numch     : %d\n",runp->numch);
       printf("   ntrack    : %d\n",runp->ntrack);
       printf("   fmhz      : %f\n",runp->fmhz);
       printf("   adbit     : %d\n",runp->adbit);
       printf("   fanout    : %d\n",runp->fanout);
       printf("   parity    : %d\n",runp->parity);
       printf("   nrzm      : %d\n",runp->nrzm);
       printf("   vlba      : %d\n",runp->vlba);
       printf("   soffset   : %d\n",runp->soffset);
       printf("   period    : %d\n",runp->period);
       printf("   odd       : %d\n",runp->odd);
       printf("   tot_track : %d\n",runp->tot_track);
       for(n=0; n<runp->tot_track; n++){
           printf("   ibit[%2d]  : %d\n",n,runp->ibit[n]);
       }
       break;
    }

    printf("\n");
}

int main(int argc, char *argv[])
{
       int i, k, n;
       int eflag;
       int iret; // 2011-02-09
       double telapsed;           // to measure processing time
       char fname[256];
       char fpath[256];
       char program_version[60];
       char* dflt_iname=INFONAME_DFLT;   // information file default name
       time_t tstart, tfinish;    
       mk5tok5_info_t mk5info;
       gr_info_t gr; 
       m5k5runp_t runp;


       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("%s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   default directory for Mark5 data\n",M5DIR);
          printf("                      ( %s )\n",getenv(M5DIR));
          printf("        %s ---   default directory for Mark5 vex file\n",M5VEX);
          printf("                      ( %s )\n",getenv(M5VEX));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* run parameters get */
       param_get_m5tok5(argc, argv, &runp); 

       if (runp.info_file[0]=='\0'){
          strcpy(runp.info_file,dflt_iname);
       }

       /* out directory create if not set */

       if(runp.outdir[0]=='\0'){
          parse_dir_and_name(runp.mk5name, runp.outdir, fname); // debug
          //printf("A %s\n",runp.mk5name);
          //printf("A %s\n",runp.outdir);
          //printf("A %s\n",fname);
       }

       /* create an information file if selected so */
       if(runp.makemode ==1){  //
           /*****************************************************/
           /* Entry of Information file creation mode operation */ 
           /*****************************************************/
           printf("M5tok5 running under Information File create mode\n");
           printf("info file (%s) will be created (updated).\n",runp.info_file);
           if(runp.vex_file[0]=='\0'){  // interactive operation
              printf("\nEnter VEX file name -> ");
              //scanf("%s",&runp.vex_file);
              iret=scanf("%s",runp.vex_file);  // 2011-02-09
           } 

          /* get true path for vex file name  */
           get_true_path(runp.vex_file, M5VEX, "./",fpath);
           strcpy(runp.vex_file,fpath);
           printf("VEX file name --- %s\n",runp.vex_file);

           make_mk5tok5_info_file(runp.info_file,runp.mk5name,
               runp.vex_file,runp.sid,runp.scan,
               program_version, runp.dflt_pass);
           printf("\nInformation file (%s) created!!\n",runp.info_file);
           exit(1);
           /***************************************************/
           /* End of Information file creation mode operation */ 
           /***************************************************/
       }

       /**************************************/
       /* Entry of Conversion mode operation */ 
       /**************************************/


       if(runp.ptype == 1){ // in case of mode 1 run parameters
           /* internal conversion information set up */
           mk5tok5_info_setup(runp.info_file, &mk5info);

           /* monitor mk5info */
           if(runp.monit==1) monit_mk5info(&mk5info);  // debug    

       } else {    // in case of mode 2 run parameters
           strcpy(runp.k5name,runp.k5name_given);
           /* set parameters for mode 2 operation */ 
           if(runp.nrzm==-1){ // automatic set this parameter
               if(runp.parity==1) runp.nrzm=1;
               if(runp.parity==0) runp.nrzm=0;
           }
           mk5info.parity=runp.parity;
           mk5info.nrzm=runp.nrzm;  
           mk5info.adbit=runp.adbit;
           mk5info.ntrack=runp.ntrack;
           mk5info.samplrate=runp.fmhz*1.0e6;    // sampling frequency (Hz)
           mk5info.fanout=runp.fanout;
           mk5info.vlba=runp.vlba;
           if(mk5info.vlba==1){
               mk5info.bitsframe=20160;
           } else {
               mk5info.bitsframe=20000;
           }
       }




       /* Program Name Display */
       printf("************************************************************\n");
       printf("*    Mark-V to K5/VSSP Data Format Converter (FAST)        *\n");
       printf("*      %s  by  T.KONDO/NICT     *\n",program_version);
       printf("************************************************************\n");
       printf("\n");

       monit_runp(&runp);

       tstart= time(NULL);


       /* Mark5 data structure check if selected so*/
       if(runp.ptype==1){  // mode 1 run parameters
          k=0;
          for(n=0; n<mk5info.ntrack; n++) k+=mk5info.bit_valid[n] ;
          if(k == 0) {  // novalid track
              printf(" No valid track was found in the information file.\n");
              goto exitend;   
          } 
       }

       gr.channel=runp.channel;
       gr.group=runp.group;
       gr.reverse=runp.reverse;


   /********************************************/
   /* loop entry of Frequency group processing */ 
   /********************************************/

loop_g:

       if(runp.ptype == 1){ // in case of mode 1 run parameters

          if(runp.channel==0 && runp.group==0) {
             gr.group++;
             if(mk5info.nch-4*(gr.group-1) <=0) goto exit_loop_g;
          }
       
          // setup processing channel number table etc.
          i=Grinfo_fm_Mk5info(&mk5info, &gr);
          if(i==-1) goto exit_loop_g;
          if((i==-2) && runp.group==0) goto loop_g;  // if all channel ==0 next group
 
          /* if odd mode is set and ntrack=64, edit bit infomation table */
          if(runp.odd==1 && mk5info.ntrack==64){
             shift_bit_trk_table(gr.bit_trk, gr.tot_track);
          } 


          /* conversion start message */
          if(runp.channel == 0){
             printf("*************************************************\n");
             printf("* Group# %2d : CH# %2d %2d %2d %2d conversion start! *\n",
               gr.group, 
               mk5info.group_channel[gr.group-1][0],
               mk5info.group_channel[gr.group-1][1],
               mk5info.group_channel[gr.group-1][2],
               mk5info.group_channel[gr.group-1][3]);
             printf("*************************************************\n");
          } else {
             printf("****************************\n");
             printf("* CH# %2d conversion start! *\n", gr.channel);
             printf("****************************\n");
          } 
       } else {  // mode 2 run parameters
           gr.numch=runp.numch;
           gr.tot_track=runp.tot_track;
           for(n=0; n<gr.tot_track; n++) gr.bit_trk[n]=runp.ibit[n];

           /* if odd mode is set and ntrack=64, edit bit infomation table */
           if(runp.odd==1 && mk5info.ntrack==64){
             shift_bit_trk_table(gr.bit_trk, gr.tot_track);
           }

           /* tot_track bit_trk table consistency check */

           eflag=0;
           for(n=0; n<gr.tot_track; n++){
             if(gr.bit_trk[n] > mk5info.ntrack-1){
               printf(" bit position +1 (%d) exceeds max track# %d\n",
                   gr.bit_trk[n], mk5info.ntrack);
               eflag=1;
             }
           }
           if(eflag==1){
              printf(" Check run parameter!\n");
              goto exitend;
           }
           // total_track must be devided by numch*adbit
           if(gr.tot_track%(gr.numch*mk5info.adbit) != 0){
               printf(" # of bit to pickup is inconsistent with # of ch (%d)\n",
                   gr.numch);
               printf("    and # of AD bit (%d)\n",mk5info.adbit);
               printf(" Check run parameter!\n");
               goto exitend;
           }

           // fan out check 

           if(mk5info.fanout!=gr.tot_track/(gr.numch*mk5info.adbit)){
               printf(" given fan out parameter (%d) is inconsistent with\n",mk5info.fanout);
               printf("   that calculated from AD bits and bit pick up table.\n");
               mk5info.fanout=gr.tot_track/(gr.numch*mk5info.adbit);
               printf(" fan out parameter is set to %d\n",mk5info.fanout);
           }

           printf("*******************************************\n");
           printf("* BitPos : %2d",gr.bit_trk[0]);
           if(gr.tot_track > 1) {
               for(n=1; n<gr.tot_track; n++) printf(" %2d",gr.bit_trk[n]);
           }
           printf("  conversion start! *\n");
           printf("*******************************************\n");

       }

       /* conversion using a function optimized to tarck # and mode*/
       if(mk5info.vlba==0){  // Mark IV mode
           if(mk5info.nrzm==0 && mk5info.parity==0){ // NRZL  non-parity
               switch(mk5info.ntrack){
               case 16: i=conv2k5_16_l(&mk5info, &gr, &runp); 
                   break;
               case 32: i=conv2k5_32_l(&mk5info, &gr, &runp); 
                   break;
               case 64: i=conv2k5_64_l(&mk5info, &gr, &runp); 
                   break;
               default: i=conv2k5(&mk5info, &gr, &runp); // general conversion
               }
               i=conv2k5(&mk5info, &gr, &runp); // general conversion
           } else if(mk5info.nrzm==1 && mk5info.parity==1){ // NRZM  with parity
               switch(mk5info.ntrack){
               case 32: i=conv2k5_32_m(&mk5info, &gr, &runp); 
                   break;
               default: i=conv2k5(&mk5info, &gr, &runp); // general conversion
               }
           } else {
               i=conv2k5(&mk5info, &gr, &runp); // general conversion
           }
       } else {  //  VLBA mode
           i=conv2k5(&mk5info, &gr, &runp); // general conversion
       }

       if(i < 0) goto exitend;

       printf("  K5 file (%s) has been created.\n\n",runp.k5name);

       if(runp.ptype==1){  // in case of Mode 1 operation
           if(runp.channel==0 && runp.group==0) goto loop_g;
       }

exit_loop_g:


       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
       printf("Time elapsed (sec) for One obs process is %f\n",telapsed);

exitend: 
       return 0;
}
